﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectExcavator.DrawingObjects;
using ProjectExcavator.Excavators;

namespace ProjectExcavator
{
    public partial class FormExcavatorConfig : Form
    {
        /// <summary>
        /// Переменная-выбранного экскаватора
        /// </summary>
        DrawingExcavator? _excavator = null;
        /// <summary>
        /// Встроенный делегат
        /// </summary>
        private event Action<DrawingExcavator>? EventAddExcavator;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormExcavatorConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (sender, e) =>
            {
                Close();
            };
        }
        /// <summary>
        /// Отрисовать экскаватор
        /// </summary>
        private void DrawExcavator()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _excavator?.SetPosition(5, 5);
            _excavator?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name,
           DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _excavator = new DrawingExcavator((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _excavator = new DrawingExcavatorKovsh((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBox_Kovsh.Checked, checkBox_Katki.Checked, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
            }
            DrawExcavator();
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawingExcavator> ev)
        {
            if (EventAddExcavator == null)
            {
                EventAddExcavator = ev;
            }
            else
            {
                EventAddExcavator += ev;
            }
        }
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_excavator == null)
                return;
            ((Label)sender).BackColor = (Color)e.Data.GetData(typeof(Color));
            switch (((Label)sender).Name)
            {
                case "labelMainColor":
                    _excavator.bodyColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "labelAdditionalColor":
                    if (!(_excavator is DrawingExcavatorKovsh))
                        return;
                    (_excavator as DrawingExcavatorKovsh).additionalColor((Color)e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawExcavator();
        }
        /// <summary>
        /// Добавление экскаватора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddExcavator?.Invoke(_excavator);
            Close();
        }
    }
}
