﻿using ProjectExcavator.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectExcavator.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
    where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="excavator">Добавляемый экскаватор</param>
        /// <returns></returns>
        public bool Insert(T excavator)
        {
            return Insert(excavator, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="excavator">Добавляемый экскаватор</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public bool Insert(T excavator, int position)
        {
            // TODO проверка позиции
            if (position < 0 || position >= _maxCount)
            {
                throw new ExcavatorNotFoundException(position);
            }
            // TODO проверка, что есть место для вставки
            if (_places.Count >= _maxCount)
            {
                throw new StorageOverflowException(_places.Count);
            }
            // TODO вставка по позиции
            _places.Insert(position, excavator);
            return true;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            // TODO проверка позиции
            if (position < 0 || position >= _places.Count)
            {
                throw new ExcavatorNotFoundException(position);
            }
            // TODO удаление объекта из списка
            _places.RemoveAt(position);
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            get
            {
                // TODO проверка позиции
                if (position < 0 || position >= _places.Count)
                {
                    return null;
                }
                return _places[position];
            }
            set
            {
                // TODO проверка позиции
                if (position < 0 || position >= _places.Count)
                {
                    return;
                }
                // TODO проверка свободных мест в списке
                if (_places.Count >= _maxCount)
                {
                    return;
                }
                // TODO вставка в список по позиции
                _places.Insert(position, value);
            }
        }
        /// <summary>
        /// Проход по списку
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T?> GetExcavators(int? maxExcavators = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxExcavators.HasValue && i == maxExcavators.Value)
                {
                    yield break;
                }
            }
        }
    }
}
